#!/usr/bin/env python3
import os
import json
from pprint import pprint
from tap_geosource.geo import GeoSource
import singer
from singer import utils, metadata
from singer.catalog import Catalog, CatalogEntry
from singer.schema import Schema
from tap_geosource.discovery import generate_schemas
from tap_geosource.sync import geo_generator


REQUIRED_CONFIG_KEYS = ['path']
LOGGER = singer.get_logger()


def discover(config):
    streams = []
    schemas = generate_schemas(config)

    for stream_id, schema in schemas.items():
        stream_metadata = []
        key_properties = []

        streams.append(
            CatalogEntry(
                tap_stream_id=stream_id,
                stream=stream_id,
                schema=schema,
                key_properties=key_properties,
                metadata=stream_metadata,
                replication_key=None,
                is_view=None,
                database=None,
                table=None,
                row_count=None,
                stream_alias=None,
                replication_method=None,
            )
        )

    return Catalog(streams)


def sync(config, state, catalog):
    """ Sync data from tap source """
    geo_source = GeoSource(path=config['path'], config=config)

    for stream in catalog.get_selected_streams(state):
        # Fetch the layer from the geo source
        layer = geo_source.layers[stream.tap_stream_id]

        LOGGER.info("Syncing stream:" + stream.tap_stream_id)

        singer.write_schema(
            stream_name=stream.tap_stream_id,
            schema=stream.schema.to_dict(),
            key_properties=stream.key_properties,
        )

        for row in layer.features():
            # write one or more rows to the stream:
            singer.write_records(stream.tap_stream_id, [row])

    # # Loop over selected streams in catalog
    # for stream in catalog.get_selected_streams(state):
    #     LOGGER.info("Syncing stream:" + stream.tap_stream_id)

    #     singer.write_schema(
    #         stream_name=stream.tap_stream_id,
    #         schema=stream.schema.to_dict(),
    #         key_properties=stream.key_properties,
    #     )

    #     for row in geo_generator(stream):
    #         # write one or more rows to the stream:
    #         singer.write_records(stream.tap_stream_id, [row])
    # return


@utils.handle_top_exception(LOGGER)
def main():
    # Parse command line arguments
    args = utils.parse_args(REQUIRED_CONFIG_KEYS)

    # If discover flag was passed, run discovery mode and dump output to stdout
    if args.discover:
        catalog = discover(args.config)
        catalog.dump()
    # Otherwise run in sync mode
    else:
        if args.catalog:
            catalog = args.catalog
        else:
            catalog = discover(args.config)
        sync(args.config, args.state, catalog)


if __name__ == "__main__":
    main()



import singer
from typing import Dict
from osgeo import gdal, ogr
from singer.schema import Schema
from pathlib import Path
from pprint import pprint

LOGGER = singer.get_logger()

# Enable GDAL/OGR exceptions
gdal.UseExceptions()


def geo_generator(stream: singer.CatalogEntry):
    # Path is stored in the stream name
    path = stream.stream

    # Open or load the geosource
    geo_file = ogr.Open(path)

    # Get the next layer of the geosource
    layer = geo_file.GetLayer()

    for i in range(layer.GetFeatureCount()):
        feature = layer.GetFeature(i)

        yield {
            **feature.items(),
            # 'geometry': str(feature.geometry())
        }



from typing import Dict, Generator, List
from osgeo import gdal, ogr, osr

# Enable GDAL/OGR exceptions
gdal.UseExceptions()

gdal.SetConfigOption('OGR_WFS_LOAD_MULTIPLE_LAYER_DEFN', 'NO')

URL = 'WFS:https://geo.leefbaarometer.nl/leefbaarometer/ows?service=WFS&version=1.0.0'
# URL = '/home/jules/Github/thesis/data/dresden/gis_osm_adminareas_a_07_1.shp'

# A mapping from ogr types to jsonschema types
OGR_DATA_TYPES = {
    0: ['null', 'integer'],
    2: ['null', 'number'],
    4: ['null', 'string'],
    12: ['null', 'integer']
}


class GeoLayer:
    def __init__(self, layer: ogr.Layer, target_srid: int) -> None:
        self.layer: ogr.Layer = layer
        self.target_srid: int = target_srid

    @property
    def name(self) -> str:
        """Get the name of the layer, we need to replace the ':' as it interferes with the bigquery table names 

        Returns:
            str: The name of the layer
        """
        return self.layer.GetName().replace(':', '_')

    @property
    def source_projection(self) -> osr.SpatialReference:
        return self.layer.GetSpatialRef()

    @property
    def target_projection(self) -> osr.SpatialReference:
        target_reference = osr.SpatialReference()
        target_reference.ImportFromEPSG(self.target_srid)

        return target_reference

    @property
    def transformer(self) -> osr.CoordinateTransformation:
        return osr.CoordinateTransformation(self.source_projection, self.target_projection)

    @property
    def schema(self) -> Dict[str, Dict[str, List[str]]]:
        layer_schema = {}

        # Loop through the fields in the schema
        for field in self.layer.schema:
            layer_schema[field.name] = {
                'type': OGR_DATA_TYPES[field.type]
            }

        # Add a default geometry field
        layer_schema['geometry'] = {
            'type': OGR_DATA_TYPES[4]
        }

        return layer_schema

    def features(self) -> Generator[Dict, None, None]:
        # Iterate over features
        feature = self.layer.GetNextFeature()
        while feature is not None:
            geometry = feature.GetGeometryRef()

            # If a re-projection is defined
            if self.target_srid:
                geometry.Transform(self.transformer)

            # Yield the properties and geometry
            yield {
                **feature.items(),
                'geometry': geometry.ExportToWkt()
            }
            # Get the next feature
            feature = self.layer.GetNextFeature()


class GeoSource:
    def __init__(self, path: str, config: Dict) -> None:
        self.path: str = path
        self.data_source: ogr.DataSource = ogr.Open(path)
        self.config: Dict = config

    @property
    def layers(self) -> Dict[str, GeoLayer]:
        target_srid = self.config.get('target_srid', None)
        include_layers = self.config.get('include_layers', None)

        # Fetch all layers
        layers = [GeoLayer(layer=layer, target_srid=target_srid)
                  for layer in self.data_source if layer]

        # If a selection of layers was made
        if include_layers:
            return {layer.name: layer for layer in layers if layer.name in include_layers}
        # Return all layers
        else:
            return {layer.name: layer for layer in layers}



#!/usr/bin/env python
from setuptools import setup
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="tap-geosource",
    version="0.1.0",
    description="Singer.io tap for extracting geo data",
    author="Jules Huisman",
    author_email="jules.huisman@quantile.nl",
    url="https://github.com/JulesHuisman/tap-geosource",
    classifiers=["Programming Language :: Python :: 3 :: Only"],
    py_modules=["tap_geosource"],
    long_description=README,
    long_description_content_type="text/markdown",
    install_requires=[
        "singer-python==5.11.0",
        "pathlib==1.0.1"
    ],
    entry_points="""
    [console_scripts]
    tap-geosource=tap_geosource:main
    """,
    packages=["tap_geosource"],
    include_package_data=True
)



import singer
from typing import Dict
from singer.schema import Schema
from tap_geosource.geo import GeoSource

LOGGER = singer.get_logger()


def generate_schemas(config: Dict) -> Dict:
    """Generate schemas based on the files or urls supplied in the config

    Args:
        config (Dict): Configuration

    Returns:
        Dict: Return a dictionary with stream_id: schema, key value pairs
    """
    # Storage place for the individual schemas
    schemas = {}

    # Create a geo datasource instance
    geo_source = GeoSource(path=config['path'], config=config)

    # Each layer is a stream
    for layer_name, layer in geo_source.layers.items():
        LOGGER.info(f'Found layer {layer_name}')
        schemas[layer_name] = Schema.from_dict({
            'type': ['null', 'object'],
            'additionalProperties': False,
            'selected': True,
            'properties': layer.schema
        })

    return schemas
